@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -258,106 +258,29 @@ static void store_dumper(void *arg)
 
 /*------------------------------------------------------*/
 
-static Octstr *store_file_status(int status_type)
+static void store_file_for_each_message(void(*callback_fn)(Msg* msg, void *data), void *data)
 {
-    char *frmt;
-    Octstr *ret, *key;
-    unsigned long l;
-    struct tm tm;
-    Msg *msg;
     List *keys;
-    char id[UUID_STR_LEN + 1];
+    long l;
+    Msg *msg;
+    Octstr *key;
 
-    ret = octstr_create("");
-
-    /* set the type based header */
-    if (status_type == BBSTATUS_HTML) {
-        octstr_append_cstr(ret, "<table border=1>\n"
-            "<tr><td>SMS ID</td><td>Type</td><td>Time</td><td>Sender</td><td>Receiver</td>"
-            "<td>SMSC ID</td><td>BOX ID</td><td>UDH</td><td>Message</td>"
-            "</tr>\n");
-    } else if (status_type == BBSTATUS_TEXT) {
-        octstr_append_cstr(ret, "[SMS ID] [Type] [Time] [Sender] [Receiver] [SMSC ID] [BOX ID] [UDH] [Message]\n");
-    }
-   
     /* if there is no store-file, then don't loop in sms_store */
     if (filename == NULL)
-        goto finish;
+        return;
 
+    mutex_lock(file_mutex);
     keys = dict_keys(sms_dict);
-
     for (l = 0; l < gwlist_len(keys); l++) {
         key = gwlist_get(keys, l);
         msg = dict_get(sms_dict, key);
         if (msg == NULL)
             continue;
-
-        if (msg_type(msg) == sms) {
-
-            if (status_type == BBSTATUS_HTML) {
-                frmt = "<tr><td>%s</td><td>%s</td>"
-                       "<td>%04d-%02d-%02d %02d:%02d:%02d</td>"
-                       "<td>%s</td><td>%s</td><td>%s</td>"
-                       "<td>%s</td><td>%s</td><td>%s</td></tr>\n";
-            } else if (status_type == BBSTATUS_XML) {
-                frmt = "<message>\n\t<id>%s</id>\n\t<type>%s</type>\n\t"
-                       "<time>%04d-%02d-%02d %02d:%02d:%02d</time>\n\t"
-                       "<sender>%s</sender>\n\t"
-                       "<receiver>%s</receiver>\n\t<smsc-id>%s</smsc-id>\n\t"
-                       "<box-id>%s</box-id>\n\t"
-                       "<udh-data>%s</udh-data>\n\t<msg-data>%s</msg-data>\n\t"
-                       "</message>\n";
-            } else {
-                frmt = "[%s] [%s] [%04d-%02d-%02d %02d:%02d:%02d] [%s] [%s] [%s] [%s] [%s] [%s]\n";
-            }
-
-            /* transform the time value */
-#if LOG_TIMESTAMP_LOCALTIME
-            tm = gw_localtime(msg->sms.time);
-#else
-            tm = gw_gmtime(msg->sms.time);
-#endif
-            if (msg->sms.udhdata)
-                octstr_binary_to_hex(msg->sms.udhdata, 1);
-            if (msg->sms.msgdata &&
-                (msg->sms.coding == DC_8BIT || msg->sms.coding == DC_UCS2 ||
-                (msg->sms.coding == DC_UNDEF && msg->sms.udhdata)))
-                octstr_binary_to_hex(msg->sms.msgdata, 1);
-
-            uuid_unparse(msg->sms.id, id);
-
-            octstr_format_append(ret, frmt, id,
-                (msg->sms.sms_type == mo ? "MO" :
-                 msg->sms.sms_type == mt_push ? "MT-PUSH" :
-                 msg->sms.sms_type == mt_reply ? "MT-REPLY" :
-                 msg->sms.sms_type == report_mo ? "DLR-MO" :
-                 msg->sms.sms_type == report_mt ? "DLR-MT" : ""),
-                 tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
-                 tm.tm_hour, tm.tm_min, tm.tm_sec,
-                (msg->sms.sender ? octstr_get_cstr(msg->sms.sender) : ""),
-                (msg->sms.receiver ? octstr_get_cstr(msg->sms.receiver) : ""),
-                (msg->sms.smsc_id ? octstr_get_cstr(msg->sms.smsc_id) : ""),
-                (msg->sms.boxc_id ? octstr_get_cstr(msg->sms.boxc_id) : ""),
-                (msg->sms.udhdata ? octstr_get_cstr(msg->sms.udhdata) : ""),
-                (msg->sms.msgdata ? octstr_get_cstr(msg->sms.msgdata) : ""));
-
-            if (msg->sms.udhdata)
-                octstr_hex_to_binary(msg->sms.udhdata);
-            if (msg->sms.msgdata &&
-                (msg->sms.coding == DC_8BIT || msg->sms.coding == DC_UCS2 ||
-                (msg->sms.coding == DC_UNDEF && msg->sms.udhdata)))
-                octstr_hex_to_binary(msg->sms.msgdata);
-        }
+        callback_fn(msg, data);
     }
+    mutex_unlock(file_mutex);
+
     gwlist_destroy(keys, octstr_destroy_item);
-
-finish:
-    /* set the type based footer */
-    if (status_type == BBSTATUS_HTML) {
-        octstr_append_cstr(ret,"</table>");
-    }
-
-    return ret;
 }
 
 
@@ -598,7 +521,7 @@ int store_file_init(const Octstr *fname, long dump_freq)
     store_load = store_file_load;
     store_dump = store_file_dump;
     store_shutdown = store_file_shutdown;
-    store_status = store_file_status;
+    store_for_each_message = store_file_for_each_message;
 
     if (fname == NULL)
         return 0; /* we are done */
