@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Kannel Software License, Version 1.0
  *
- * Copyright (c) 2001-2014 Kannel Group
+ * Copyright (c) 2001-2018 Kannel Group
  * Copyright (c) 1998-2001 WapIT Ltd.
  * All rights reserved.
  *
@@ -60,7 +60,7 @@
  * This 'generic' type will handle the 'send-url' directive in the
  * group the same way the 'sms-service' for smsbox does, via
  * URLTranslation. Response interpretation is based on the three
- * regex value that match against the reponse body. The HTTP reponse
+ * regex value that match against the response body. The HTTP response
  * code is not obeyed.
  *
  * Example config group:
@@ -396,17 +396,18 @@ static void generic_receive_sms(SMSCConn *conn, HTTPClient *client,
         debug("smsc.http.generic", 0, "HTTP[%s]: Constructing new SMS",
               octstr_get_cstr(conn->id));
 
-        /* convert character encoding if required */
+        msg->sms.msgdata = octstr_duplicate(text);
+
+        /* re-encode content if necessary */
         if (conndata->alt_charset &&
-            charset_convert(text, octstr_get_cstr(conndata->alt_charset),
-                    DEFAULT_CHARSET) != 0)
-            error(0, "Failed to convert msgdata from charset <%s> to <%s>, will leave it as it is.",
-                    octstr_get_cstr(conndata->alt_charset), DEFAULT_CHARSET);
+                sms_charset_processing(conndata->alt_charset, msg->sms.msgdata, msg->sms.coding) == -1) {
+            error(0, "HTTP[%s]: Charset or body misformed, will leave it as it is.",
+                  octstr_get_cstr(conn->id));
+        }
 
         msg->sms.service = octstr_duplicate(user);
         msg->sms.sender = octstr_duplicate(from);
         msg->sms.receiver = octstr_duplicate(to);
-        msg->sms.msgdata = octstr_duplicate(text);
         msg->sms.udhdata = octstr_duplicate(udh);
         msg->sms.smsc_id = octstr_duplicate(conn->id);
         msg->sms.time = time(NULL);
@@ -517,7 +518,7 @@ static void generic_parse_reply(SMSCConn *conn, Msg *msg, int status,
             SMSCCONN_FAILED_TEMPORARILY, octstr_duplicate(body));
     }
     else {
-        error(0, "HTTP[%s]: Message was rejected. SMSC reponse was:",
+        error(0, "HTTP[%s]: Message was rejected. SMSC response was:",
               octstr_get_cstr(conn->id));
         octstr_dump(body, 0);
         bb_smscconn_send_failed(conn, msg,
@@ -531,12 +532,6 @@ static int generic_init(SMSCConn *conn, CfgGroup *cfg)
     ConnData *conndata = conn->data;
     struct generic_values *values;
 
-    /* we need at least the criteria for a successful sent */
-    if ((os = cfg_get(cfg, octstr_imm("status-success-regex"))) == NULL) {
-        error(0, "HTTP[%s]: 'status-success-regex' required for generic http smsc",
-              octstr_get_cstr(conn->id));
-        return -1;
-    }
     conndata->data = values = gw_malloc(sizeof(*values));
     /* reset */
     memset(conndata->data, 0, sizeof(*values));
@@ -547,11 +542,16 @@ static int generic_init(SMSCConn *conn, CfgGroup *cfg)
     values->map = generic_get_field_map(cfg);
 
     /* pre-compile regex expressions */
-    if (os != NULL) {   /* this is implicit due to the above if check */
+    if ((os = cfg_get(cfg, octstr_imm("status-success-regex"))) != NULL) {
         if ((values->success_regex = gw_regex_comp(os, REG_EXTENDED|REG_NOSUB)) == NULL)
             error(0, "HTTP[%s]: Could not compile pattern '%s' defined for variable 'status-success-regex'",
                   octstr_get_cstr(conn->id), octstr_get_cstr(os));
         octstr_destroy(os);
+    } else {
+        /* we need at least the criteria for a successful sent */
+        error(0, "HTTP[%s]: 'status-success-regex' required for generic http smsc",
+              octstr_get_cstr(conn->id));
+        return -1;
     }
     if ((os = cfg_get(cfg, octstr_imm("status-permfail-regex"))) != NULL) {
         if ((values->permfail_regex = gw_regex_comp(os, REG_EXTENDED|REG_NOSUB)) == NULL)
