@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -76,6 +76,7 @@ struct smpp_tlv {
     Octstr *name;
     long tag;
     long length;
+    Octstr *constant;
     enum { SMPP_TLV_OCTETS = 0, SMPP_TLV_NULTERMINATED = 1, SMPP_TLV_INTEGER = 2 } type;
 };
 
@@ -83,6 +84,7 @@ struct smpp_tlv {
 static Dict *tlvs_by_tag;
 /* Dict(smsc_id, Dict(tag_name, tlv)) */
 static Dict *tlvs_by_name;
+static Dict *tlvs_by_const;
 static List *tlvs;
 static int initialized;
 
@@ -92,9 +94,41 @@ static void smpp_tlv_destroy(struct smpp_tlv *tlv)
     if (tlv == NULL)
         return;
     octstr_destroy(tlv->name);
+    octstr_destroy(tlv->constant);
     gw_free(tlv);
 }
 
+
+void smpp_tlv_add_constant(Octstr *smsc_id, Dict **tlvs)
+{
+    Dict *constant;
+    List *keys;
+    Octstr *key;
+    struct smpp_tlv *tlv;
+
+    constant = dict_get(tlvs_by_const, smsc_id);
+    if (constant == NULL)
+        return;
+
+    keys = dict_keys(constant);
+    while ((key = gwlist_extract_first(keys)) != NULL) {
+        /* only add if not available already */
+        tlv = dict_get(constant, key);
+        if (tlv && tlv->constant) {
+            if (*tlvs) {
+                if (dict_get(*tlvs, key) == NULL)
+                    dict_put(*tlvs, key, octstr_duplicate(tlv->constant));
+            } else {
+                *tlvs = dict_create(1024, octstr_destroy_item);
+                dict_put(*tlvs, key, octstr_duplicate(tlv->constant));
+            }
+        }
+        octstr_destroy(key);
+    }
+    gwlist_destroy(keys, NULL);
+}
+
+
 static struct smpp_tlv *smpp_tlv_get_by_name(Octstr *smsc_id, Octstr *name)
 {
     struct smpp_tlv *res = NULL;
@@ -159,6 +193,7 @@ int smpp_pdu_init(Cfg *cfg)
     tlvs = gwlist_create();
     tlvs_by_tag = dict_create(1024, (void(*)(void*))dict_destroy);
     tlvs_by_name = dict_create(1024, (void(*)(void*))dict_destroy);
+    tlvs_by_const = dict_create(1024, (void(*)(void*))dict_destroy);
     while (l != NULL && (grp = gwlist_extract_first(l)) != NULL) {
         struct smpp_tlv *tlv;
         Octstr *tmp, *smsc_id;
@@ -170,6 +205,7 @@ int smpp_pdu_init(Cfg *cfg)
             smpp_tlv_destroy(tlv);
             goto failed;
         }
+        tlv->constant = cfg_get(grp, octstr_imm("const"));
         if (cfg_get_integer(&tlv->tag, grp, octstr_imm("tag")) == -1) {
             error(0, "SMPP: Unable to get tag for smpp-tlv.");
             smpp_tlv_destroy(tlv);
@@ -215,6 +251,7 @@ int smpp_pdu_init(Cfg *cfg)
 
             debug("sms.smpp", 0, "adding smpp-tlv for smsc-id=%s", octstr_get_cstr(smsc_id));
 
+            /* name */
             tmp_dict = dict_get(tlvs_by_name, smsc_id);
             if (tmp_dict == NULL) {
                 tmp_dict = dict_create(1024, NULL);
@@ -227,6 +264,7 @@ int smpp_pdu_init(Cfg *cfg)
                 goto failed;
             }
 
+            /* tag */
             tmp_dict = dict_get(tlvs_by_tag, smsc_id);
             if (tmp_dict == NULL) {
                 tmp_dict = dict_create(1024, NULL);
@@ -240,6 +278,21 @@ int smpp_pdu_init(Cfg *cfg)
                 octstr_destroy(smsc_id);
                 goto failed;
             }
+
+            /* const */
+            if (tlv->constant != NULL) {
+                tmp_dict = dict_get(tlvs_by_const, smsc_id);
+                if (tmp_dict == NULL) {
+                    tmp_dict = dict_create(1024, NULL);
+                    dict_put(tlvs_by_const, smsc_id, tmp_dict);
+                }
+                if (!dict_put_once(tmp_dict, tlv->name, tlv)) {
+                    error(0, "SMPP: Double TLV name %s found.", octstr_get_cstr(tlv->name));
+                    octstr_destroy(smsc_id);
+                    goto failed;
+                }
+            }
+
             octstr_destroy(tmp);
             octstr_destroy(smsc_id);
         }
@@ -319,6 +372,30 @@ static int copy_until_nul(const char *field_name, Octstr *os, long *pos, long ma
     return 0;
 }
 
+static int is_defined_field(long type, const char *field_name)
+{
+    switch (type) {
+    #define OPTIONAL_BEGIN
+    #define TLV_INTEGER(name, octets) OCTETS(name, octets)
+    #define TLV_NULTERMINATED(name, max_len) OCTETS(name, max_len)
+    #define TLV_OCTETS(name, min_len, max_len) OCTETS(name, min_len)
+    #define OPTIONAL_END
+    #define INTEGER(name, octets) OCTETS(name, octets)
+    #define NULTERMINATED(name, max_octets) OCTETS(name, max_octets)
+    #define OCTETS(name, field_giving_octetst) \
+            if (strncmp(#name, field_name, sizeof(#name) - 1) == 0) \
+                return 1;
+    #define PDU(name, id, fields) \
+        case id: { \
+            fields \
+        } break;
+    #include "smpp_pdu.def"
+    default:
+        break;
+    }
+
+    return 0;
+}
 
 SMPP_PDU *smpp_pdu_create(unsigned long type, unsigned long seq_no)
 {
@@ -442,7 +519,8 @@ Octstr *smpp_pdu_pack(Octstr *smsc_id, SMPP_PDU *pdu)
             while(keys != NULL && (key = gwlist_extract_first(keys)) != NULL) { \
                 tlv = smpp_tlv_get_by_name(smsc_id, key); \
                 if (tlv == NULL) { \
-                    error(0, "SMPP: Unknown TLV `%s', don't send.", octstr_get_cstr(key)); \
+                    if (!is_defined_field(pdu->type, octstr_get_cstr(key))) \
+                        error(0, "SMPP: Unknown TLV `%s', don't send.", octstr_get_cstr(key)); \
                     octstr_destroy(key); \
                     continue; \
                 } \
@@ -573,6 +651,11 @@ SMPP_PDU *smpp_pdu_unpack(Octstr *smsc_id, Octstr *data_without_len)
                         pos += opt_len; \
                         continue; \
                     } \
+                    if(p->mname != NULL) { \
+                        warning(0, "SMPP: Optional field (%s) was sent more than once, overwriting", #mname); \
+                        octstr_destroy(p->mname); \
+                        p->mname = NULL; \
+                    } \
                     copy_until_nul(#mname, data_without_len, &pos, opt_len, &p->mname); \
                     if (tlv != NULL) dict_put(p->tlv, tlv->name, octstr_duplicate(p->mname)); \
                 } else
@@ -585,6 +668,11 @@ SMPP_PDU *smpp_pdu_unpack(Octstr *smsc_id, Octstr *data_without_len)
                         pos += opt_len; \
                         continue; \
                     } \
+                    if(p->mname != NULL) { \
+                        warning(0, "SMPP: Optional field (%s) was sent more than once, overwriting", #mname); \
+                        octstr_destroy(p->mname); \
+                        p->mname = NULL; \
+                    } \
                     p->mname = octstr_copy(data_without_len, pos, opt_len); \
                     pos += opt_len; \
                     if (tlv != NULL) dict_put(p->tlv, tlv->name, octstr_duplicate(p->mname)); \
@@ -725,6 +813,59 @@ void smpp_pdu_dump(Octstr *smsc_id, SMPP_PDU *pdu)
 }
 
 
+void smpp_pdu_dump_line(Octstr *smsc_id, SMPP_PDU *pdu)
+{
+    Octstr *str = octstr_create("");
+
+    octstr_format_append(str, "SMPP PDU %p dump: [type_name:%d:%s]", (void *) pdu, strlen(pdu->type_name), pdu->type_name);
+    switch (pdu->type) {
+    #define OPTIONAL_BEGIN
+    #define TLV_INTEGER(name, max_len) \
+        if (p->name != -1)  { \
+            INTEGER(name, max_len) \
+        }
+    #define TLV_NULTERMINATED(name, max_len) \
+        if (p->name != NULL) { \
+            NULTERMINATED(name, max_len) \
+        }
+    #define TLV_OCTETS(name, min_len, max_len) \
+        if (p->name != NULL) { \
+            OCTETS(name, max_len) \
+        }
+    #define OPTIONAL_END \
+        if (p->tlv != NULL) { \
+            List *keys; \
+            Octstr *key; \
+            struct smpp_tlv *tlv; \
+            keys = dict_keys(p->tlv); \
+            while(keys != NULL && (key = gwlist_extract_first(keys)) != NULL) { \
+                tlv = smpp_tlv_get_by_name(smsc_id, key); \
+                if (tlv != NULL) { \
+                    Octstr *val = dict_get(p->tlv, key); \
+                    octstr_format_append(str, " [%E:%d:%E]", key, octstr_len(val), val); \
+                } \
+                octstr_destroy(key); \
+            } \
+            gwlist_destroy(keys, octstr_destroy_item); \
+        }
+    #define INTEGER(name, octets) \
+        octstr_format_append(str, " [%s:0:0x%08lx]", #name, p->name);
+    #define NULTERMINATED(name, max_octets) \
+        octstr_format_append(str, " [%s:%d:%E]", #name, octstr_len(p->name), (p->name != NULL ? p->name : octstr_imm("NULL")));
+    #define OCTETS(name, field_giving_octets) \
+        octstr_format_append(str, " [%s:%d:%E]", #name, octstr_len(p->name), (p->name != NULL ? p->name : octstr_imm("NULL")));
+    #define PDU(name, id, fields) \
+        case id: { struct name *p = &pdu->u.name; fields } break;
+    #include "smpp_pdu.def"
+    default:
+        error(0, "Unknown SMPP_PDU type, internal error.");
+        break;
+    }
+    debug("sms.smpp", 0, "%s", octstr_get_cstr(str));
+    octstr_destroy(str);
+}
+
+
 long smpp_pdu_read_len(Connection *conn)
 {
     Octstr *os;
@@ -780,7 +921,7 @@ const char *smpp_error_to_string(enum SMPP_ERROR_MESSAGES error)
             return "Invalid Command ID";
         case SMPP_ESME_RINVBNDSTS:
             return "Incorrect BIND Status for given command";
-        case SMPP_ESME_RALYNBD:
+        case SMPP_ESME_RALYBND:
             return "ESME Already in Bound State";
         case SMPP_ESME_RINVPRTFLG:
             return "Invalid Priority Flag";
