@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -128,7 +128,8 @@
 #include "urltrans.h"
 #include "meta_data.h"
 
-#define DEFAULT_CHARSET "UTF-8"
+#define DEFAULT_CHARSET         "UTF-8"
+#define DEFAULT_UCS2_CHARSET    "UTF-16BE"
 
 /* callback functions set by HTTP-SMSC type */
 struct smsc_http_fn_callbacks {
@@ -632,7 +633,7 @@ static void kannel_receive_sms(SMSCConn *conn, HTTPClient *client,
                                List *headers, Octstr *body, List *cgivars)
 {
     ConnData *conndata = conn->data;
-    Octstr *user, *pass, *from, *to, *text, *udh, *account, *binfo;
+    Octstr *user, *pass, *from, *to, *text, *udh, *account, *binfo, *charset;
     Octstr *dlrmid, *dlrerr;
     Octstr *tmp_string, *retmsg;
     int	mclass, mwi, coding, validity, deferred, dlrmask;
@@ -648,6 +649,7 @@ static void kannel_receive_sms(SMSCConn *conn, HTTPClient *client,
     to = http_cgi_variable(cgivars, "to");
     text = http_cgi_variable(cgivars, "text");
     udh = http_cgi_variable(cgivars, "udh");
+    charset = http_cgi_variable(cgivars, "charset");
     account = http_cgi_variable(cgivars, "account");
     binfo = http_cgi_variable(cgivars, "binfo");
     dlrmid = http_cgi_variable(cgivars, "dlr-mid");
@@ -769,11 +771,21 @@ static void kannel_receive_sms(SMSCConn *conn, HTTPClient *client,
         msg->sms.deferred = (deferred == SMS_PARAM_UNDEFINED ? deferred : time(NULL) + deferred * 60);
         msg->sms.account = octstr_duplicate(account);
         msg->sms.binfo = octstr_duplicate(binfo);
-        ret = bb_smscconn_receive(conn, msg);
-        if (ret == -1)
-            retmsg = octstr_create("Not accepted");
-        else
-            retmsg = octstr_create("Sent.");
+
+        /* re-encode content if necessary */
+        if (sms_charset_processing(charset, msg->sms.msgdata, msg->sms.coding) == -1) {
+            error(0, "HTTP[%s]: Charset or body misformed, rejected",
+                  octstr_get_cstr(conn->id));
+            retmsg = octstr_create("Charset or body misformed, rejected");
+        }
+        else {
+
+            ret = bb_smscconn_receive(conn, msg);
+            if (ret == -1)
+                retmsg = octstr_create("Not accepted");
+            else
+                retmsg = octstr_create("Sent.");
+        }
     }
 
     reply_headers = gwlist_create();
@@ -808,11 +820,34 @@ static int httpsmsc_send(SMSCConn *conn, Msg *msg)
 
     sms = msg_duplicate(msg);
     /* convert character encoding if required */
-    if (msg->sms.coding == DC_7BIT && conndata->alt_charset &&
-        charset_convert(sms->sms.msgdata, DEFAULT_CHARSET,
-                        octstr_get_cstr(conndata->alt_charset)) != 0) {
-        error(0, "Failed to convert msgdata from charset <%s> to <%s>, will send as is.",
-              DEFAULT_CHARSET, octstr_get_cstr(conndata->alt_charset));
+    if (conndata->alt_charset) {
+        /*
+         * Converted now to the target character set based on the
+         * one we got in the msg, which is either UTF-8 (our normal
+         * inter-box encoding), but may also be UCS-2, so beware.
+         * In addition, IF we convert to an "extra" encoding here
+         * we also revert the .coding vaue to DC_UNDEF, in order
+         * that all API specific code doesn't indicate an encoding
+         * which is no longer inside the payload here.
+         */
+        if (sms->sms.coding == DC_7BIT) {
+            if (charset_convert(sms->sms.msgdata, DEFAULT_CHARSET,
+                    octstr_get_cstr(conndata->alt_charset)) == 0) {
+                sms->sms.coding = DC_UNDEF;
+            } else {
+                error(0, "Failed to convert msgdata from charset <%s> to <%s>, will send as is.",
+                         DEFAULT_CHARSET, octstr_get_cstr(conndata->alt_charset));
+            }
+        }
+        else if (sms->sms.coding == DC_UCS2) {
+            if (charset_convert(sms->sms.msgdata, DEFAULT_UCS2_CHARSET,
+                    octstr_get_cstr(conndata->alt_charset)) == 0) {
+               sms->sms.coding = DC_UNDEF;
+            } else {
+                error(0, "Failed to convert msgdata from charset <%s> to <%s>, will send as is.",
+                        DEFAULT_UCS2_CHARSET, octstr_get_cstr(conndata->alt_charset));
+            }
+        }
     }
 
     gwlist_produce(conndata->msg_to_send, sms);
@@ -825,8 +860,10 @@ static long httpsmsc_queued(SMSCConn *conn)
 {
     ConnData *conndata = conn->data;
 
-    return (conndata ? (conn->status != SMSCCONN_DEAD ? 
-            counter_value(conndata->open_sends) : 0) : 0);
+    conn->load = (conndata ? (conn->status != SMSCCONN_DEAD ?
+            gwlist_len(conndata->msg_to_send) : 0) : 0);
+
+    return conn->load;
 }
 
 
@@ -862,7 +899,6 @@ static int httpsmsc_shutdown(SMSCConn *conn, int finish_sending)
 #include "http/brunet.c"
 #include "http/xidris.c"
 #include "http/clickatell.c"
-#include "http/wapme.c"
 
 
 int smsc_http_create(SMSCConn *conn, CfgGroup *cfg)
@@ -934,8 +970,6 @@ int smsc_http_create(SMSCConn *conn, CfgGroup *cfg)
         conndata->callbacks = &smsc_http_generic_callback;
     } else if (octstr_case_compare(type, octstr_imm("clickatell")) == 0) {
         conndata->callbacks = &smsc_http_clickatell_callback;
-    } else if (octstr_case_compare(type, octstr_imm("wapme")) == 0) {
-        conndata->callbacks = &smsc_http_wapme_callback;
     }
     /*
      * ADD NEW HTTP SMSC TYPES HERE
