@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -381,7 +381,7 @@ Octstr *urltrans_fill_escape_codes(Octstr *pattern, Msg *request)
         }
         break;
 
-    case 'b':
+    case 'b':   /* message payload, URL-encoded */
         enc = octstr_duplicate(request->sms.msgdata);
         octstr_url_encode(enc);
         octstr_append(result, enc);
@@ -437,6 +437,15 @@ Octstr *urltrans_fill_escape_codes(Octstr *pattern, Msg *request)
         }
         break;
 
+    case 'e':   /* message payload, printable hexadecimal byte codes */
+        if (octstr_len(request->sms.msgdata)) {
+            enc = octstr_duplicate(request->sms.msgdata);
+            octstr_binary_to_hex(enc, 1);
+            octstr_append(result, enc);
+            octstr_destroy(enc);
+        }
+        break;
+
     case 'f':  /* smsc number*/
         if (octstr_len(request->sms.smsc_number)) {
             enc = octstr_duplicate(request->sms.smsc_number);
