@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -64,7 +64,7 @@
  * Jarkko Kovala <jarkko.kovala@iki.fi>
  *
  * SSL server implementation contributed by
- * Stipe Tolj <stolj@wapme.de> for Wapme Systems AG
+ * Stipe Tolj <stolj at kannel.org>
  */
 
 /* TODO: unlocked_close() on error */
@@ -84,6 +84,8 @@
 #ifdef HAVE_LIBSSL
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#include <openssl/conf.h>
+#include <openssl/engine.h>
 
 static SSL_CTX *global_ssl_context = NULL;
 static SSL_CTX *global_server_ssl_context = NULL;
@@ -249,6 +251,11 @@ static long unlocked_write(Connection *conn)
             } else {
                 error(errno, "SSL write failed: OpenSSL error %d: %s",
                       SSL_error, ERR_error_string(SSL_error, NULL));
+                if (SSL_error == SSL_ERROR_SSL) { /* trace library errors */
+                    long err;
+                    while ((err = ERR_get_error()) != 0) 
+                        error(0, "SSL %s", ERR_error_string(err, NULL));
+                }
                 return -1;
             }
         }
@@ -1268,6 +1275,8 @@ static RSA *tmp_rsa_callback(SSL *ssl, int export, int key_len)
 }
 */
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+
 static Mutex **ssl_static_locks = NULL;
 
 /* the call-back function for the openssl crypto thread locking */
@@ -1310,8 +1319,14 @@ void openssl_shutdown_locks(void)
     ssl_static_locks = NULL;
 }
 
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+
 void conn_init_ssl(void)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    openssl_init_locks();
+#endif
+
     SSL_library_init();
     SSL_load_error_strings();
     global_ssl_context = SSL_CTX_new(SSLv23_client_method());
@@ -1333,11 +1348,20 @@ void server_ssl_init(void)
 
 void conn_shutdown_ssl(void)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    openssl_shutdown_locks();
+#endif
+
     if (global_ssl_context)
         SSL_CTX_free(global_ssl_context);
     
+    CONF_modules_free();
+    ERR_remove_state(0);
+    ENGINE_cleanup();
+    CONF_modules_unload(1);
     ERR_free_strings();
     EVP_cleanup();
+    CRYPTO_cleanup_all_ex_data();
 }
 
 void server_shutdown_ssl(void)
@@ -1345,15 +1369,19 @@ void server_shutdown_ssl(void)
     if (global_server_ssl_context)
         SSL_CTX_free(global_server_ssl_context);
 
+    CONF_modules_free();
+    ERR_remove_state(0);
+    ENGINE_cleanup();
+    CONF_modules_unload(1);
     ERR_free_strings();
     EVP_cleanup();
+    CRYPTO_cleanup_all_ex_data();
 }
 
-void use_global_client_certkey_file(Octstr *certkeyfile)
+void conn_use_global_client_certkey_file(Octstr *certkeyfile)
 { 
-    SSL_CTX_use_certificate_file(global_ssl_context, 
-                                 octstr_get_cstr(certkeyfile), 
-                                 SSL_FILETYPE_PEM);
+    SSL_CTX_use_certificate_chain_file(global_ssl_context, 
+                                       octstr_get_cstr(certkeyfile));
     SSL_CTX_use_PrivateKey_file(global_ssl_context,
                                 octstr_get_cstr(certkeyfile),
                                 SSL_FILETYPE_PEM);
@@ -1365,11 +1393,10 @@ void use_global_client_certkey_file(Octstr *certkeyfile)
          octstr_get_cstr(certkeyfile));
 }
 
-void use_global_server_certkey_file(Octstr *certfile, Octstr *keyfile) 
+void conn_use_global_server_certkey_file(Octstr *certfile, Octstr *keyfile) 
 {
-    SSL_CTX_use_certificate_file(global_server_ssl_context, 
-                                  octstr_get_cstr(certfile), 
-                                  SSL_FILETYPE_PEM);
+    SSL_CTX_use_certificate_chain_file(global_server_ssl_context, 
+                                       octstr_get_cstr(certfile));
     SSL_CTX_use_PrivateKey_file(global_server_ssl_context,
                                  octstr_get_cstr(keyfile),
                                  SSL_FILETYPE_PEM);
@@ -1384,14 +1411,42 @@ void use_global_server_certkey_file(Octstr *certfile, Octstr *keyfile)
     info(0, "Using global server SSL key from file `%s'", octstr_get_cstr(keyfile));
 }
 
+void conn_use_global_client_cipher_list(Octstr *cipher)
+{
+    if (SSL_CTX_set_cipher_list(global_ssl_context, octstr_get_cstr(cipher)) != 1) {
+        error(0, "SSL: %s", ERR_error_string(ERR_get_error(), NULL));
+        SSL_CTX_free(global_ssl_context);
+        panic(0, "cipher list <%s> contains no supported ciphers!",
+              octstr_get_cstr(cipher));
+    }
+}
+
+void conn_use_global_server_cipher_list(Octstr *cipher)
+{
+    if (SSL_CTX_set_cipher_list(global_server_ssl_context, octstr_get_cstr(cipher)) != 1) {
+        error(0, "SSL: %s", ERR_error_string(ERR_get_error(), NULL));
+        SSL_CTX_free(global_server_ssl_context);
+        panic(0, "cipher list <%s> contains no supported ciphers!",
+              octstr_get_cstr(cipher));
+    }
+}
+
 static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 {
     char    subject[256];
     char    issuer [256];
     char   *status;
+    X509   *curr_cert;
 
-    X509_NAME_oneline(X509_get_subject_name(ctx->current_cert), subject, sizeof(subject));
-    X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), issuer, sizeof (issuer));
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    curr_cert = ctx->current_cert;
+    X509_NAME_oneline(X509_get_subject_name(curr_cert), subject, sizeof(subject));
+    X509_NAME_oneline(X509_get_issuer_name(curr_cert), issuer, sizeof (issuer));
+#else
+    curr_cert = X509_STORE_CTX_get_current_cert(ctx);
+    X509_NAME_oneline(X509_get_subject_name(curr_cert), subject, sizeof(subject));
+    X509_NAME_oneline(X509_get_issuer_name(curr_cert), issuer, sizeof (issuer));
+#endif
 
     status = preverify_ok ? "Accepting" : "Rejecting";
     
@@ -1400,34 +1455,36 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
     return preverify_ok;
 }
 
-void use_global_trusted_ca_file(Octstr *ssl_trusted_ca_file)
+void conn_use_global_trusted_ca_file(Octstr *ssl_trusted_ca_file)
 {
     if (ssl_trusted_ca_file != NULL) {
-	if (!SSL_CTX_load_verify_locations(global_ssl_context,
-					   octstr_get_cstr(ssl_trusted_ca_file),
-					   NULL)) {
-	    panic(0, "Failed to load SSL CA file: %s", octstr_get_cstr(ssl_trusted_ca_file));
-	} else {
-	    info(0, "Using CA root certificates from file %s",
-		 octstr_get_cstr(ssl_trusted_ca_file));
-	    SSL_CTX_set_verify(global_ssl_context,
-			       SSL_VERIFY_PEER,
-			       verify_callback);
-	}
-	
+        if (!SSL_CTX_load_verify_locations(global_ssl_context,
+                octstr_get_cstr(ssl_trusted_ca_file),
+                NULL)) {
+            panic(0, "Failed to load SSL CA file: %s", octstr_get_cstr(ssl_trusted_ca_file));
+        } else {
+            info(0, "Using CA root certificates from file %s",
+                    octstr_get_cstr(ssl_trusted_ca_file));
+            SSL_CTX_set_verify(global_ssl_context,
+                    SSL_VERIFY_PEER,
+                    verify_callback);
+        }
+
     } else {
-	SSL_CTX_set_verify(global_ssl_context,
-			   SSL_VERIFY_NONE,
-			   NULL);
+        SSL_CTX_set_verify(global_ssl_context,
+                SSL_VERIFY_NONE,
+                NULL);
     }
 }
 
 void conn_config_ssl (CfgGroup *grp)
 {
     Octstr *ssl_client_certkey_file = NULL;
-    Octstr *ssl_server_cert_file    = NULL;
-    Octstr *ssl_server_key_file     = NULL;
-    Octstr *ssl_trusted_ca_file     = NULL;
+    Octstr *ssl_server_cert_file = NULL;
+    Octstr *ssl_server_key_file = NULL;
+    Octstr *ssl_trusted_ca_file = NULL;
+    Octstr *ssl_client_cipher_list = NULL;
+    Octstr *ssl_server_cipher_list = NULL;
 
     /*
      * check if SSL is desired for HTTP servers and then
@@ -1436,24 +1493,36 @@ void conn_config_ssl (CfgGroup *grp)
      */    
     ssl_client_certkey_file = cfg_get(grp, octstr_imm("ssl-client-certkey-file"));
     if (ssl_client_certkey_file != NULL) 
-        use_global_client_certkey_file(ssl_client_certkey_file);
+        conn_use_global_client_certkey_file(ssl_client_certkey_file);
     
     ssl_server_cert_file = cfg_get(grp, octstr_imm("ssl-server-cert-file"));
     ssl_server_key_file = cfg_get(grp, octstr_imm("ssl-server-key-file"));
     
     if (ssl_server_cert_file != NULL && ssl_server_key_file != NULL) {
-        use_global_server_certkey_file(ssl_server_cert_file, 
+        conn_use_global_server_certkey_file(ssl_server_cert_file, 
 				       ssl_server_key_file);
     }
 
     ssl_trusted_ca_file = cfg_get(grp, octstr_imm("ssl-trusted-ca-file"));
     
-    use_global_trusted_ca_file(ssl_trusted_ca_file);
+    conn_use_global_trusted_ca_file(ssl_trusted_ca_file);
+
+    /*
+     * Check if specific ciphers are selected/de-selected.
+     */
+    if ((ssl_client_cipher_list = cfg_get(grp, octstr_imm("ssl-client-cipher-list"))) != NULL) {
+        conn_use_global_client_cipher_list(ssl_client_cipher_list);
+    }
+    if ((ssl_server_cipher_list = cfg_get(grp, octstr_imm("ssl-server-cipher-list"))) != NULL) {
+        conn_use_global_server_cipher_list(ssl_server_cipher_list);
+    }
 
     octstr_destroy(ssl_client_certkey_file);
     octstr_destroy(ssl_server_cert_file);
     octstr_destroy(ssl_server_key_file);
     octstr_destroy(ssl_trusted_ca_file);
+    octstr_destroy(ssl_client_cipher_list);
+    octstr_destroy(ssl_server_cipher_list);
 }
 
 SSL *conn_get_ssl(Connection *conn)
