@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -88,7 +88,7 @@ int main(int argc, char **argv)
     unsigned char macbuf[EVP_MAX_MD_SIZE], *p;
     int mac_len;
 #ifdef HAVE_LIBSSL
-    HMAC_CTX ctx;
+    HMAC_CTX *ctx;
 #endif
 
     gwlib_init();
@@ -109,11 +109,19 @@ int main(int argc, char **argv)
     octstr_dump(data, 0);
 
 #ifdef HAVE_LIBSSL
-    HMAC_Init(&ctx, octstr_get_cstr(key), octstr_len(key), EVP_sha1());
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    HMAC_Init(ctx, octstr_get_cstr(key), octstr_len(key), EVP_sha1());
+#else
+    ctx = HMAC_CTX_new();
+#endif
     p = HMAC(EVP_sha1(), octstr_get_cstr(key), octstr_len(key), 
          octstr_get_cstr(data), octstr_len(data), 
          macbuf, &mac_len);
-    HMAC_cleanup(&ctx);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    HMAC_cleanup(ctx);
+#else
+    HMAC_CTX_free(ctx);
+#endif
 #else
     macbuf[0] = 0;
     mac_len = 0;
