@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 2001-2018 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -87,6 +87,7 @@ static Octstr *method_name = NULL;
 static int file = 0;
 static List *split = NULL;
 static int follow_redirect = 1;
+static int escape_codes = 0;
 
 
 static Octstr *post_content_create(void)
@@ -101,6 +102,62 @@ static Octstr *post_content_create(void)
     return content;
 }
 
+
+static void url_pattern(Octstr *url)
+{
+    Octstr *temp;
+    const char *pattern;
+    size_t n;
+
+    temp = octstr_duplicate(url);
+    octstr_truncate(url, 0);
+
+    pattern = octstr_get_cstr(temp);
+
+    while (*pattern != '\0') {
+        n = strcspn(pattern, "%");
+        octstr_append_data(url, pattern, n);
+        pattern += n;
+        gw_assert(*pattern == '%' || *pattern == '\0');
+        if (*pattern == '\0')
+            break;
+
+        pattern++;
+
+        switch (*pattern) {
+
+        case 'r':
+            octstr_format_append(url, "%ld", gw_rand());
+            break;
+
+        case 'I':
+        {
+            uuid_t uid;
+            char id[UUID_STR_LEN + 1];
+            uuid_generate(uid);
+            uuid_unparse(uid, id);
+            octstr_append_cstr(url, id);
+        }
+            break;
+
+            /* XXX add more here if needed */
+
+        case '%':
+            octstr_format_append(url, "%%");
+            break;
+
+        default:
+            warning(0, "Unknown escape code (%%%c) within URL, skipping!", *pattern);
+                octstr_format_append(url, "%%%c", *pattern);
+            break;
+        } /* switch(...) */
+
+        pattern++;
+    } /* while ... */
+
+    octstr_destroy(temp);
+}
+
 static void start_request(HTTPCaller *caller, List *reqh, long i)
 {
     Octstr *url, *content = NULL;
@@ -128,7 +185,11 @@ static void start_request(HTTPCaller *caller, List *reqh, long i)
         content = post_content_create();
         method = HTTP_METHOD_POST;
     }
-                                
+
+    /* apply any escape codes */
+    if (escape_codes)
+        url_pattern(url);
+
     /*
      * if this is a POST request then pass the required content as body to
      * the HTTP server, otherwise skip the body, the arguments will be
@@ -286,6 +347,8 @@ static void help(void)
     info(0, "    use `domain.name' as a proxy");
     info(0, "-P portnumber");
     info(0, "    connect to proxy at port `portnumber'");
+    info(0, "-E regex");
+    info(0, "    proxy exceptions as regex value");
     info(0, "-S");
     info(0, "    use HTTPS scheme to access SSL-enabled proxy server");
     info(0, "-e domain1:domain2:...");
@@ -305,8 +368,13 @@ static void help(void)
     info(0, "    use HTTPS scheme to access SSL-enabled HTTP server");
     info(0, "-c ssl_client_cert_key_file");
     info(0, "    use this file as the SSL certificate and key file");
+    info(0, "-C ssl_ca_file");
+    info(0, "    use this file as the SSL certificate authority");
     info(0, "-f");
     info(0, "    don't follow redirects");
+    info(0, "-V");
+    info(0, "    evaluate for URL escape code patterns (%r - random number,");
+    info(0, "    %I - UUID string)");
 }
 
 int main(int argc, char **argv) 
@@ -325,6 +393,7 @@ int main(int argc, char **argv)
     double run_time;
     FILE *fp;
     int ssl = 0;
+    Octstr *ca_file;
     
     gwlib_init();
     
@@ -338,7 +407,7 @@ int main(int argc, char **argv)
     file = 0;
     fp = NULL;
     
-    while ((opt = getopt(argc, argv, "hv:qr:p:P:Se:t:i:a:u:sc:H:B:m:f")) != EOF) {
+    while ((opt = getopt(argc, argv, "hv:qr:p:P:Se:t:i:a:u:sc:H:B:m:fVC:")) != EOF) {
 	switch (opt) {
 	case 'v':
 	    log_set_output_level(atoi(optarg));
@@ -448,6 +517,18 @@ int main(int argc, char **argv)
         follow_redirect = 0;
         break;
 
+    case 'V':
+        escape_codes = 1;
+        break;
+
+#ifdef HAVE_LIBSSL
+    case 'C':
+        ca_file = octstr_create(optarg);
+        conn_use_global_trusted_ca_file(ca_file);
+        octstr_destroy(ca_file);
+        break;
+#endif
+
     case '?':
 	default:
 	    error(0, "Invalid option %c", opt);
@@ -468,7 +549,7 @@ int main(int argc, char **argv)
      */
     if (ssl || proxy_ssl) {
         if (ssl_client_certkey_file != NULL) {
-            use_global_client_certkey_file(ssl_client_certkey_file);
+            conn_use_global_client_certkey_file(ssl_client_certkey_file);
         } else {
             panic(0, "client certkey file need to be given!");
         }
